//
//  CantripPlugin.h
//  CantripPluginSDK
//
//  Created by James Callender on Wed Apr 24 2002.
//  Copyright (c) 2001 James Callender. All rights reserved.
//

/*! @header CantripPlugin.h
    @discussion Description forthcoming.  To things to note right now: World is a subclass 
	of NSDocument, and WorldWindowController is a subclass of NSWindowController.
*/

#import <Cocoa/Cocoa.h>
#import "CantripSettingsModule.h"

@class World, WorldWindowController, Socket, VTEmulationView, CantripSettingsModule;
@protocol WorldProtocol, WorldWindowControllerProtocol;

/*!
	@typedef WorldConnectionState
	State values for a world connection window.
    @abstract Description forthcoming.
    @discussion Description forthcoming.
    @constant WorldStartState State value before any connections have been made or attempted.
    @constant WorldConnectingState State value while a connection is being attempted.
    @constant WorldConnectedState State value while a connection is active.
	@constant WorldDisconnectedState State value after a connection has been dropped.
*/
typedef enum _WorldConnectionState
{
    WorldStartState,
    WorldConnectingState,
    WorldConnectedState,
	WorldDisconnectedState
} WorldConnectionState;


/*!
	@typedef CantripPluginService
	Constants for various plugin services.
    @abstract Description forthcoming.
    @discussion Description forthcoming.
    @constant InterfaceEventService Interface event services.  Only keyEvents supported currently.
    @constant ConnectionEventService Connection event services.
    @constant DirtyServerStreamService Service for intercepting incoming server data.
	@constant CleanServerStreamService Service for intercepting cleaned server data.  ANSI color codes and other VT
	emulation codes have been stripped.
	@constant ClientStreamService Service for intercepting user entered data.
*/
typedef enum _CantripPluginService
{
	InterfaceEventService,
	ConnectionEventService,
	DirtyServerStreamService,
//	CleanServerStreamService,
	AttributedServerStreamService,
//	ClientStreamService,
	ScriptStreamService
} CantripPluginService;


/*!
	@typedef WorldConnectionEvent
	Constants for connection events.
    @abstract Description forthcoming.
    @discussion Discussion forthcoming.
    @constant ConnectionDidDrop Description forthcoming.
    @constant ConnectionUnsuccessful Description forthcoming.
    @constant ConnectionTerminated Description forthcoming.
	@constant ConnectionLoggedOff Description forthcoming.
*/
typedef enum _WorldConnectionEvent
{
	ConnectionSuccessful,
	ConnectionDidDrop,
	ConnectionUnsuccessful,
	ConnectionTerminated,
	ConnectionLoggedOff
} WorldConnectionEvent;


/*!
	@typedef CantripHandleResponse
	Constants for service handling responses.
    @abstract Description forthcoming.
    @discussion Discussion forthcoming.
    @constant ContinueNoChangeResponse Description forthcoming.
    @constant StopNoChangeResponse Description forthcoming.
    @constant ContinueChangedResponse Description forthcoming.
	@constant StopChangedResponse Description forthcoming.
*/
typedef enum _CantripHandleResponse
{
	ContinueNoChangeResponse,
	ContinueWithChangeResponse,
	StopNoChangeResponse,
	StopWithChangeResponse,
} CantripHandleResponse;


/*!
 	@protocol WorldProtocol
	@abstract Protocol implemented by World object.
	@discussion Description forthcoming. 
*/
@protocol WorldProtocol

/*!
	@method activeController
	@abstract Returns the active window controller.
	@discussion This method will return the active (frontmost) window controller
	for the document.  Will return nil if none is present.
*/
- (id <WorldWindowControllerProtocol>)activeController;

/*!
	@method worldData
	@abstract Returns NSMutableDictionary object containing world-specific data
	@discussion This is the mutable dictionary object that stores all the world data.
	Although there is no real guidelines (currently) you should follow when
	storing data here, your safest bet is to create a dictionary within this
	that has a unique key value for your plugin, so as to avoid naming comflicts.
*/
- (NSMutableDictionary*)worldData;

@end

/*!
 	@protocol WorldWindowProtocol
	@abstract Protocol implemented by WorldWindowController object.
	@discussion Description forthcoming. 
*/

@protocol WorldWindowControllerProtocol

/*!
	@method controllerID
	@abstract Returns a unique identifier for the controller.
	@discussion Discussion forthcoming.
*/
- (NSString *)controllerID;

/*!
	@method socket
	@abstract Returns the socket object for the target.
	@discussion This object will be an instance of a SmallSockets class Socket.
*/
- (Socket *)socket;

/*!
	@method state
	@abstract Returns an enumerated value describing the state of the connection.
	@discussion Description forthcoming.
*/
- (WorldConnectionState)state;

/*!
	@method termView
	@abstract Returns the terminal emulation view.  
	@discussion A method description for this class will be written shortly.
*/
- (VTEmulationView*)termView;

/*!
	@method setState:
	@abstract Sets the world state.  
	@discussion Note: this method will not update the window for the new state.
	@param s New state for connection window.
*/
- (void)setState:(WorldConnectionState)s;

/*!
	@method setStateAndUpdate:
	@abstract Sets the world state and updates the interface.  
	@discussion Description forthcoming.
	@param s New state for connection window.
*/
- (void)setStateAndUpdate:(WorldConnectionState)s;

/*!
	@method setMsgBar:
	@abstract Set the message bar.  
	@discussion Description forthcoming.
	@param msg NSString object to be placed in the message bar.
*/
- (void)setMsgBar:(NSString *)msg;

/*!
	@method connect
	@abstract Attempt a connection.  
	@discussion This method will attempt a connection using the current worlds "WorldAddress"
	and "WorldKey" entries in the world data dictionary.
*/
- (void)connect;

/*!
	@method disconnect
	@abstract Disconnect from server.  
	@discussion Description forthcoming.
*/
- (void)disconnect;

/*!
	@method sendString:
	@abstract Send a string to the server.  
	@discussion Description forthcoming.
	@param string String to be sent to server.
*/
- (void)sendString:(NSString *)string;


/*!
	@method sendStringAndEcho:
	@abstract Send a string to the server and echo it.  
	@discussion Description forthcoming.
	@param string String to be sent to server.
*/
- (void)sendStringAndEcho:(NSString *)string;

/*!
	@method echoString:
	@abstract Echo a string to the window's output.  
	@discussion Description forthcoming.
	@param string String to be echoed.
*/
- (void)echoString:(NSString *)string;

/*!
	@method callScript:
	@abstract Calls a script to be handled by other plugins and then sent to server.  
	@discussion Description forthcoming.
	@param string String to be called.
*/
- (void)callScript:(NSString *)string;

@end

/*!
 	@class CantripPlugin
	@abstract CantripPlugin provides a Cantrip Plugin class.
	@discussion Description forthcoming. 
*/

@interface CantripPlugin : NSObject
{

}

/*!
	@method initializeClass:
	@abstract Initializes plugin class.  
	@discussion Perform any global initializations such as checking its environment, preloading icons and preferences, and so forth..=
*/
+ (BOOL)initializeClass:(NSBundle*)theBundle;

/*!
	@method terminateClass
	@abstract Terminate plugin class.  
	@discussion This is called when the plug-in won't be needed anymore, just before the application terminates, and should close or release all resources opened or retained by initializeClass:.

*/
+ (void)terminateClass;

/*!
	@method pluginName
	@abstract Returns the unique identifier for the plugin.  
	@discussion Subclasses must override this.
*/
- (NSString *)pluginName;

/*!
	@method pluginVersion
	@abstract Returns the version string for the plugin.  
	@discussion Overriding this is optional.
*/
- (NSString *)pluginVersion;

/*!
	@method settingsModule
	@abstract Returns a settings module for the plugin, if one exists.  
	@discussion Returns nil by default.
*/
- (CantripSettingsModule *)settingsModule;

/*!
	@method applicationDidLoadPlugin
	@abstract Called when Cantrip loads the plugin at program launch.  
	@discussion Overriding is optional but recommended.
*/
- (void)applicationDidLoadPlugin;

/*!
	@method applicationDidCreateUntitledWorld:
	@abstract Called when a new world object was created.  
	@discussion These methods are called for various actions concerning world creation
	and initialization.  Overriding applicationDidCreateUntitledWorld: is useful
	for initializing default data in the world document.	
	@param world Pointer to the new world object.
*/
- (void)applicationDidCreateUntitledWorld:(id <WorldProtocol>)world;


/*!
	@method applicationDidLoadWorld:
	@abstract Called when a world was loaded from file.  
	@discussion see applicationDidCreateUntitledWorld:.
	@param world Pointer to the new world object.
*/
- (void)applicationDidLoadWorld:(id <WorldProtocol>)world;

/*!
	@method applicationWillCloseWorld:
	@abstract Called just before a world file is closed.  
	@discussion see applicationDidCreateUntitledWorld:.
	@param world Pointer to the world object.
*/
- (void)applicationWillCloseWorld:(id <WorldProtocol>)world;

/*!
	@method applicationDidCloseWorld:
	@abstract Called just after world file is closed.  
	@discussion see applicationDidCreateUntitledWorld:.
	@param world Pointer to the world object.
*/
- (void)applicationDidCloseWorld:(id <WorldProtocol>)world;

/*!
	@method applicationWillSaveWorld:
	@abstract Called just before a world is saved to file.  
	@discussion This is a good place to save any last minute data like window positions.
	@param world Pointer to the world object.
*/
- (void)applicationWillSaveWorld:(id <WorldProtocol>)world;

/*!
	@method windowControllerDidInitialize:
	@abstract Called just after a world window was initialized.  
	@discussion This is a good place to load any objects for a given window controller.
	@param controller Pointer to the window controller object.
*/
- (void)windowControllerDidInitialize:(id <WorldWindowControllerProtocol>)controller;

/*!
	@method windowControllerWillClose:
	@abstract Called just before a world window is closed.  
	@discussion This is a good place destroy any objects specific to a world window.
	@param controller Pointer to the window controller object.
*/
- (void)windowControllerWillClose:(id <WorldWindowControllerProtocol>)controller;

/*!
	@method handleInterfaceEventAction:
	@abstract Handle events for a world window.  
	@discussion Semi-experimental.  We'll see if there are any problems with this.
	This gets called when there is an event in a world connection window.
	A return of YES tells Cantrip to stop handling this event, but NO tells Cantrip
	to keep passing the event down the plugin-chain.
	@param event Event object.
*/
- (BOOL)handleInterfaceEventAction:(NSEvent *)event;

/*!
	@method handleConnectionEventAction:
	@abstract Handle connection events for a world window.  
	@discussion This gets called to handle connection events for a world window.
	@param event Event object.
*/
- (void)handleConnectionEventAction:(int)event controller:(id <WorldWindowControllerProtocol>)controller;

/*!
	@method handleDirtyDataChunk:forWorldWindow:
	@abstract Handle input data from the user before it is sent out.  
	@discussion Dicussion forthcoming.
	@param data Pointer to NSMutableData object recieved from server.
	@param controller Pointer to world window controller object.
*/
- (CantripHandleResponse)handleDirtyDataChunk:(NSMutableData *)data forWorldWindow:(id <WorldWindowControllerProtocol>)controller;

/*!
	@method handleCleanDataChunk:forWorldWindow:
	@abstract Handle input data from the user before it is sent out.  
	@discussion Dicussion forthcoming.
	@param data Pointer to NSMutableData object recieved from server.
	@param controller Pointer to world window controller object.
*/
- (CantripHandleResponse)handleCleanDataChunk:(NSMutableData *)data forWorldWindow:(id <WorldWindowControllerProtocol>)controller;

/*!
	@method handleAttributedDataChunk:forWorldWindow:
	@abstract Handle colored text just before drawing.  
	@discussion Dicussion forthcoming.
	@param data Pointer to NSMutableAttributedString object recieved from server.
	@param controller Pointer to world window controller object.
*/
- (CantripHandleResponse)handleAttributedDataChunk:(NSMutableAttributedString *)data forWorldWindow:(id <WorldWindowControllerProtocol>)controller;


/*	This has been removed.
	@method handleUserInput:forWorldWindow:
	@abstract Handle input data from the user before it is sent out.  
	@discussion For handling incoming data from the server or from the user.  Returning NO for
	either of these methods prevents Cantrip from handling the data further.
	NOTE: THIS IS EXPERIMENTAL.  My plan is to implement a much more flexible
	system that will be more favorable to "plugin-chains".
	@param input Input NSMutableString to be handled.
	@param controller Pointer to world window controller object.
/
- (CantripHandleResponse)handleUserInput:(NSMutableString *)input forWorldWindow:(WorldWindowController <WorldWindowControllerProtocol>*)controller;
*/

/*!
	@method handleScript:forWorldWindow:fromInput:echoString:
	@abstract Handle input data from any input source (plugin, user, etc).  
	@discussion For handling input data from plugins or from the user input.
	@param input Input NSMutableString to be handled.
	@param controller Pointer to world window controller object.
	@param fromInput YES if input originated from the standard input box.
	@param echoString Mutable string that will be echoed to output (if settings permit).  May be Nil if no echo is desired.
*/
- (CantripHandleResponse)handleScript:(NSMutableString *)input 
					   forWorldWindow:(id <WorldWindowControllerProtocol>)controller 
							fromInput:(BOOL)fromInput
						   echoString:(NSMutableString *)echoString;

/*!
	@method toolbar:itemForItemIdentifier:willBeInsertedIntoToolbar:controller:
	@abstract Standard toolbar protocol.  
	@discussion Returns a desired toolbar to be added, or nil if not handled.
	@param toolbar Pointer to the toolbar.
	@param itemIdent String identifier for toolbar item.
	@param willBeInserted Whether the toolbar item is to be inserted into the toolbar.
	@param controller Pointer to window controller.
*/
- (NSToolbarItem *)		toolbar:(NSToolbar *)toolbar 
		  itemForItemIdentifier:(NSString *)itemIdent 
	  willBeInsertedIntoToolbar:(BOOL) willBeInserted 
					 controller:(id <WorldWindowControllerProtocol>)controller;

/*!
	@method toolbarDefaultItemIdentifiers:controller:
	@abstract Standard toolbar protocol.  
	@discussion Currently not supported, sorta pointless.
	@param toolbar Pointer to NSMutableData object recieved from server.
	@param controller Pointer to window controller.
*/
- (NSArray *)toolbarDefaultItemIdentifiers:(NSToolbar *)toolbar 
				controller:(id <WorldWindowControllerProtocol>)controller;

/*!
	@method toolbarAllowedItemIdentifiers:controller:
	@abstract Standard toolbar protocol.  
	@discussion Dicussion forthcoming.
	@param toolbar Pointer to NSMutableData object recieved from server.
	@param controller Pointer to window controller.
*/
- (NSArray *)toolbarAllowedItemIdentifiers:(NSToolbar *)toolbar 
				controller:(id <WorldWindowControllerProtocol>)controller;

/*!
	@method toolbarWillAddItem:controller:
	@abstract Standard toolbar protocol.  
	@discussion Dicussion forthcoming.
	@param toolbar Pointer to NSMutableData object recieved from server.
	@param controller Pointer to window controller.
*/				
- (void)toolbarWillAddItem:(NSNotification *)notification
				controller:(id <WorldWindowControllerProtocol>)controller;

/*!
	@method toolbarDidRemoveItem:controller:
	@abstract Standard toolbar protocol.  
	@discussion Dicussion forthcoming.
	@param toolbar Pointer to NSMutableData object recieved from server.
	@param controller Pointer to window controller.
*/
- (void)toolbarDidRemoveItem:(NSNotification *)notification
				controller:(id <WorldWindowControllerProtocol>)controller;

@end

/*
	@function CantripPluginManager
	Returns plugin manager instance.
*/
extern id CantripPluginManager();

/*
	@function ActiveWorld
	Returns active world instance.
*/
extern id ActiveWorld();

/*
	@function ActiveWorldWindowController
	Returns active world window (Experimental, avoid usage).
*/
extern id ActiveWorldWindowController();
